@@ -94,7 +94,23 @@ class xtable:
         csvfile = os.path.expanduser(csvfile)
         with open(csvfile,newline='') as cf :
             reader = csv.reader(cf, delimiter=delimiter, quotechar=quotechar)
-            data += [[c for c in r] for r in [row for row in reader]]
+            data += [[c for c in r] for r in reader if r ]
+        if not header and len(data)>0:
+            header = data[0]
+            data = data[1:]
+        return xtable(data,header)
+
+    @staticmethod
+    def init_from_csv_fh(csvfh, xheader=None, delimiter=',', quotechar='"'):
+        header=list()
+        data=list()
+        if xheader :
+            if type(xheader) is list :
+                header = xheader
+            else :
+                header = [ h for h in xheader.split(",") if h]
+        reader = csv.reader(csvfh, delimiter=delimiter, quotechar=quotechar)
+        data += [[c for c in r] for r in reader if r]
         if not header and len(data)>0:
             header = data[0]
             data = data[1:]
@@ -148,6 +164,24 @@ class xtable:
             tbl.append(row)
         return yaml.safe_dump(tbl, default_flow_style=False)
 
+    def markdown(self):
+        if not self.__header  or not self.__data :
+            return ""
+        res = ""
+        width = [int(0) for _ in range(len(self.__header))]
+        for r in ([self.__header] + self.__data) :
+            for i, c in enumerate(r) :
+                if len(str(c)) > int(width[i]) :
+                    width[i] = len(c)
+        width = [max(4,w) for w in width]
+        fmtstr = "| " + "|".join([" {:"+str(w)+"} " for w in width]) + " |"
+        res += (fmtstr.format(*self.__header)) + "\n"
+        res += (fmtstr.format(*(['-'*w for w in width]))) + "\n"
+        for r in self.__data :
+            if r :
+                res += (fmtstr.format(*r)) + "\n"
+        return res
+
     def csv(self):
         import io
         si = io.StringIO()
@@ -300,12 +334,30 @@ def xtable_main():
     parser.add_argument( "-w", "--maxcolwidth", dest="maxcolwidth", type=int, default=-1, help="max col width when print in console, min==20",)
     parser.add_argument( "-t", "--table", dest="table", action="store_true", default=False, help="input preformatted by spaces. header should not include spaces.",)
     parser.add_argument( "-v", "--pivot", dest="pivot", action="store_true", default=False, help="pivot wide tables.",)
-    parser.add_argument( "-T", "--tree", dest="tree", action="store_true", default=False, help="indicate the table is of tree struture",)
-    parser.add_argument("-F", "--format", dest="format", help="json,yaml,csv,html")
-    parser.add_argument( "-d", "--dataonly", dest="dataonly", action="store_true", default=False, help="indicate there's no header",)
+    parser.add_argument("-F", "--tgtformat", dest="format", help="json,yaml,csv,html or md(markdown)")
     parser.add_argument( "-X", "--debug", dest="debug", action="store_true", default=False, help="debug mode",)
+    parser.add_argument( "--forcecsv", dest="forcecsv", action="store_true", default=False, help="treat input as CSV",)
+    parser.add_argument( "--dataonly", dest="dataonly", action="store_true", default=False, help="indicate there's no header",)
+    parser.add_argument( "--tree", dest="tree", action="store_true", default=False, help="indicate the table is of tree struture",)
     args = parser.parse_args()
 
+    def showres(t) :
+        if args.pivot:
+            print(t.pivot())
+        else:
+            if args.format == "json":
+                print(t.json(), end="")
+            elif args.format == "yaml":
+                print(t.yaml(), end="")
+            elif args.format == "csv":
+                print(t.csv(), end="")
+            elif args.format == "html":
+                print(t.html(), end="")
+            elif args.format in ["md","markdown"]:
+                print(t.markdown(), end="")
+            else:
+                print(t, end="")
+
     colsdict = defaultdict()
     colsdict_revert = defaultdict()
     if args.column:
@@ -332,8 +384,19 @@ def xtable_main():
     lno = 0
     INPUT = sys.stdin
     if args.infile:
-        with open(args.infile, "r") as f:
-            INPUT = f.readlines()
+        INPUT = open(args.infile, "r") 
+
+    if args.forcecsv :
+        done = False
+        try :
+            xt = xtable.init_from_csv_fh(INPUT)
+            showres(xt)
+            done = True
+        except :
+            traceback.print_exc()
+            pass
+        if done :
+            sys.exit(0)
 
     instr = INPUT.read()
     js = None
@@ -341,13 +404,11 @@ def xtable_main():
         js = json.loads(instr)
     except :
         INPUT = instr.splitlines()
+
     if js :
         if type(js) is list :
             xt = xtable.init_from_json(js,args.header)
-            if args.pivot :
-                print(xt.pivot())
-            else :
-                print(xt)
+            showres(xt)
         sys.exit(0)
 
     for ln in INPUT:
@@ -397,29 +458,16 @@ def xtable_main():
                 v.append(v0)
             return v
         data = sorted(data, key=fsort)
-
-    if args.pivot:
-        print(xtable(header=oheader, data=data, cols=args.dumpcols).pivot())
-    else:
-        t = xtable(
-            header=oheader,
-            data=data,
-            cols=args.dumpcols,
-            maxcolwidth=args.maxcolwidth,
-            noheader=args.dataonly,
-            tree=args.tree,
-        )
-        if args.format == "json":
-            print(t.json(), end="")
-        elif args.format == "yaml":
-            print(t.yaml(), end="")
-        elif args.format == "csv":
-            print(t.csv(), end="")
-        elif args.format == "html":
-            print(t.html(), end="")
-        else:
-            print(t, end="")
-
+    xt = xtable(
+                header=oheader,
+                data=data,
+                cols=args.dumpcols,
+                maxcolwidth=args.maxcolwidth,
+                noheader=args.dataonly,
+                tree=args.tree,
+            )
+    showres(xt)
+    
 
 if __name__ == "__main__":
     xtable_main()
